/*!\file
 * \brief    ,          .
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#ifndef SYSTEM_H_
#define SYSTEM_H_

#include "chip_5023BC016.h"
#include "types.h"



extern const uint32_t sys_freq; 
extern const uint32_t spw_freq;

#define CLK_INIT_PAUSE       for(int i=0; i<200000; i++)
#define CLK_FPU_PAUSE        for(int i=0; i<100; i++)
#define CLK_UART_PAUSE       for(int i=0; i<10000; i++)

/**
 *\brief   
 */
typedef struct
{
    __RW uint32_t INTR_MUX_CTRL_Watchdog;                                       /**<  ,  WatchDog*/
    __RW uint32_t INTR_MUX_CTRL_Timer1;                                         /**<  ,  Timer1*/
    __RW uint32_t INTR_MUX_CTRL_Timer2;                                         /**<  ,  Timer2*/
    __RW uint32_t INTR_MUX_CTRL_Timer3;                                         /**<  ,  Timer3*/
    __RW uint32_t INTR_MUX_CTRL_Timer4;                                         /**<  ,  Timer4*/
    __RW uint32_t RESERVED[3];                                                  /**<  */
    __RW uint32_t INTR_MUX_CTRL_GPIOA;                                          /**<  ,  GPIOA*/
    __RW uint32_t INTR_MUX_CTRL_GPIOB;                                          /**<  ,  GPIOB*/                                                  
    __RW uint32_t INTR_MUX_CTRL_GPIOC;                                          /**<  ,  GPIOC*/
    __RW uint32_t INTR_MUX_CTRL_GPIOD;                                          /**<  ,  GPIOD*/
    __RW uint32_t INTR_MUX_CTRL_GPIOE;                                          /**<  ,  GPIOE*/
    __RW uint32_t INTR_MUX_CTRL_GPIOF;                                          /**<  ,  GPIOF*/
    __RW uint32_t INTR_MUX_CTRL_GPIOG;                                          /**<  ,  GPIOG*/
    __RW uint32_t INTR_MUX_CTRL_GPIOH;                                          /**<  ,  GPIOH*/
    __RW uint32_t INTR_MUX_CTRL_GPIOI;                                          /**<  ,  GPIOI*/
    __RW uint32_t RESERVED1[7];                                                 /**<  */
    __RW uint32_t INTR_MUX_CTRL_UART1;                                          /**<  ,  UART1*/
    __RW uint32_t INTR_MUX_CTRL_UART2;                                          /**<  ,  UART2*/
    __RW uint32_t INTR_MUX_CTRL_UART3;                                          /**<  ,  UART3*/
    __RW uint32_t INTR_MUX_CTRL_UART4;                                          /**<  ,  UART4*/
    __RW uint32_t INTR_MUX_CTRL_UART5;                                          /**<  ,  UART5*/
    __RW uint32_t INTR_MUX_CTRL_UART6;                                          /**<  ,  UART6*/
    __RW uint32_t RESERVED2[2];                                                 /**<  */
    __RW uint32_t INTR_MUX_CTRL_SPI1;                                           /**<  ,  SPI1*/
    __RW uint32_t INTR_MUX_CTRL_SPI2;                                           /**<  ,  SPI2*/
    __RW uint32_t RESERVED3[2];                                                 /**<  */
    __RW uint32_t INTR_MUX_CTRL_SPACEWIRE1;                                     /**<  ,  SPACEWIRE1*/
    __RW uint32_t INTR_MUX_CTRL_SPACEWIRE2;                                     /**<  ,  SPACEWIRE2*/
    __RW uint32_t RESERVED4[2];                                                 /**<  */
    __RW uint32_t INTR_MUX_CTRL_MKPD1;                                          /**<  ,  MKPD1*/
    __RW uint32_t INTR_MUX_CTRL_MKPD2;                                          /**<  ,  MKPD2*/
    __RW uint32_t INTR_MUX_CTRL_MKPD3;                                          /**<  ,  MKPD3*/
    __RW uint32_t INTR_MUX_CTRL_MKPD4;                                          /**<  ,  MKPD4*/
    __RW uint32_t RESERVED5[4];                                                 /**<  */
    __RW uint32_t INTR_MUX_CTRL_TMTX;                                           /**<  ,  TMTX*/
    __RW uint32_t INTR_MUX_CTRL_TCRX;                                           /**<  ,  TCRX*/
    __RW uint32_t INTR_MUX_CTRL_I2C;                                            /**<  ,  I2C*/
    __RW uint32_t RESERVED6[1];                                                 /**<  */
    __RW uint32_t INTR_MUX_CTRL_CAN1;                                           /**<  ,  CAN1*/
    __RW uint32_t INTR_MUX_CTRL_CAN2;                                           /**<  ,  CAN2*/
    __RW uint32_t RESERVED7[2];                                                 /**<  */
    __RW uint32_t INTR_MUX_CTRL_EDAC;                                           /**<  ,  EDAC*/
    __RW uint32_t RESERVED8[3];                                                 /**<  */
    __RW uint32_t INTR_MUX_CTRL_RES_Q;                                          /**< */
    __RW uint32_t RESERVED9[3];                                                 /**<  */
    __RW uint32_t INTR_MUX_CTRL_RES_P1;                                         /**< */
    __RW uint32_t INTR_MUX_CTRL_RES_P2;                                         /**< */
    __RW uint32_t INTR_MUX_CTRL_RES_P3;                                         /**< */
    __RW uint32_t INTR_MUX_CTRL_RES_P4;                                         /**< */
    __RW uint32_t RESERVED10[28];                                               /**<  */
    __RW uint32_t INTR_MUX_CTRL_DMASPI1_TX;                                     /**<  ,  DMASPI1_TX*/
    __RW uint32_t INTR_MUX_CTRL_DMASPI1_RX;                                     /**<  ,  DMASPI1_RX*/
    __RW uint32_t INTR_MUX_CTRL_DMASPI2_TX;                                     /**<  ,  DMASPI2_TX*/
    __RW uint32_t INTR_MUX_CTRL_DMASPI2_RX;                                     /**<  ,  DMASPI2_RX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART1_TX;                                    /**<  ,  DMAUART1_TX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART1_RX;                                    /**<  ,  DMAUART1_RX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART2_TX;                                    /**<  ,  DMAUART2_TX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART2_RX;                                    /**<  ,  DMAUART2_RX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART3_TX;                                    /**<  ,  DMAUART3_TX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART3_RX;                                    /**<  ,  DMAUART3_RX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART4_TX;                                    /**<  ,  DMAUART4_TX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART4_RX;                                    /**<  ,  DMAUART4_RX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART5_TX;                                    /**<  ,  DMAUART5_TX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART5_RX;                                    /**<  ,  DMAUART5_RX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART6_TX;                                    /**<  ,  DMAUART6_TX*/
    __RW uint32_t INTR_MUX_CTRL_DMAUART6_RX;                                    /**<  ,  DMAUART6_RX*/
}CONTROL_MUX_REGISTER;

/**
 * @brief   INTR_MUX_CTRL
 */
#define INTR_SRC_ENABLE                 ((uint32_t)(1<<31))                     /**< 1     */
#define NUM_INTR_VECTOR(x)              ((uint32_t)((x&0x1F)<<0))               /**<     0  31,      */   


/** \brief      NVIC
 */
typedef enum 
{
    Watchdog             = 0,                                                   /**<   Watchdog */
    Timer1               = 1,                                                   /**<   Timer1 */
    Timer2               = 2,                                                   /**<   Timer2*/
    Timer3               = 3,                                                   /**<   Timer3 */
    Timer4               = 4,                                                   /**<   Timer4*/
    GPIOA                = 8,                                                   /**<   GPIO A */
    GPIOB                = 9,                                                   /**<   GPIO B */
    GPIOC                = 10,                                                  /**<   GPIO C */
    GPIOD                = 11,                                                  /**<   GPIO D */
    GPIOE                = 12,                                                  /**<   GPIO E */
    GPIOF                = 13,                                                  /**<   GPIO F */
    GPIOG                = 14,                                                  /**<   GPIO G */
    GPIOH                = 15,                                                  /**<   GPIO H */
    GPIOI                = 16,                                                  /**<   GPIO I */
    UART_1               = 24,                                                  /**<   UART1 */ 
    UART_2               = 25,                                                  /**<   UART2 */ 
    UART_3               = 26,                                                  /**<   UART3*/
    UART_4               = 27,                                                  /**<   UART4 */
    UART_5               = 28,                                                  /**<   UART5 */
    UART_6               = 29,                                                  /**<   UART6 */ 
    SPI_1                = 32,                                                  /**<   SPI 1 */ 
    SPI_2                = 33,                                                  /**<   SPI 2 */ 
    Spacewire1           = 36,                                                  /**<   Spacewire1 */ 
    Spacewire2           = 37,                                                  /**<   Spacewire2 */ 
    MKPD1_NUM            = 40,                                                  /**<   MKPD1 */ 
    MKPD2_NUM            = 41,                                                  /**<   MKPD2 */ 
    MKPD3_NUM            = 42,                                                  /**<   MKPD3 */ 
    MKPD4_NUM            = 43,                                                  /**<   MKPD4 */ 
    TMTX_NUM             = 48,                                                  /**<   TMTX */ 
    TCRX_NUM             = 49,                                                  /**<   TCRX */ 
    I2C_NUM              = 50,                                                  /**<   I2C */ 
    CAN1_NUM             = 52,                                                  /**<   CAN1 */ 
    CAN2_NUM             = 53,                                                  /**<   CAN2 */ 
    EDAC                 = 56,                                                  /**<   EDAC */ 
    RES_Q_NUM            = 60,                                                  /**<  */ 
    RESERVE_P1_NUM       = 64,                                                  /**<  */ 
    RESERVE_P2_NUM       = 65,                                                  /**< */ 
    RESERVE_P3_NUM       = 66,                                                  /**<  */ 
    RESERVE_P4_NUM       = 67,                                                  /**<  */ 
    DMA_SPI_1_TX         = 96,                                                  /**<   DMA_SPI_1_TX */ 
    DMA_SPI_1_RX         = 97,                                                  /**<   DMA_SPI_1_TX */ 
    DMA_SPI_2_TX         = 98,                                                  /**<   DMA_SPI_2_TX */ 
    DMA_SPI_2_RX         = 99,                                                  /**<   DMA_SPI_2_RX */ 
    DMA_UART_1_TX        = 100,                                                 /**<   DMA_UART_1_TX */ 
    DMA_UART_1_RX        = 101,                                                 /**<   DMA_UART_1_RX */ 
    DMA_UART_2_TX        = 102,                                                 /**<   DMA_UART_2_TX */ 
    DMA_UART_2_RX        = 103,                                                 /**<   DMA_UART_2_RX */ 
    DMA_UART_3_TX        = 104,                                                 /**<   DMA_UART_3_TX */ 
    DMA_UART_3_RX        = 105,                                                 /**<   DMA_UART_3_RX */ 
    DMA_UART_4_TX        = 106,                                                 /**<   DMA_UART_4_TX */ 
    DMA_UART_4_RX        = 107,                                                 /**<   DMA_UART_4_RX */ 
    DMA_UART_5_TX        = 108,                                                 /**<   DMA_UART_5_TX */ 
    DMA_UART_5_RX        = 109,                                                 /**<   DMA_UART_5_RX */ 
    DMA_UART_6_TX        = 110,                                                 /**<   DMA_UART_6_TX */ 
    DMA_UART_6_RX        = 111,                                                 /**<   DMA_UART_6_RX */                    
} IRQn_T;

/** \brief        
 */
typedef struct 
{
    __RW uint32_t EXTMEM_CTRL;                                                  /**< : 0x000     */
    __RW uint32_t EDAC_CTRL;                                                    /**< : 0x004         */
    __RO  uint32_t INTMEM_CERR_CNT;                                             /**< : 0x008      */
    __RO  uint32_t INTMEM_FERR_CNT;                                             /**< : 0x00C      */
    __RO  uint32_t EXTMEM_CERR_CNT;                                             /**< : 0x010      */
    __RO  uint32_t EXTMEM_FERR_CNT;                                             /**< : 0x014      */
       uint32_t RESERVED;
    __RW uint32_t SPACEWIRE_CLK_CTRL;                                           /**< : 0x01C     Spacewire */
    __RO uint32_t INTMEM2_CERR_CNT;                                             /**< : 0x020       2*/
    __RO uint32_t INTMEM2_FERR_CNT;                                             /**< : 0x024       2*/
    __RW uint32_t PWR_CTRL_CLK;                                                 /**< : 0x028      */
    __RW uint32_t PWR_CTRL_RST;                                                 /**< : 0x02C    */
    __RW uint32_t EXTMEM2_CTRL;                                                 /**< : 0x030      Chip Select 2*/
    __RW uint32_t EXTMEM3_CTRL;                                                 /**< : 0x034      Chip Select 3*/
    __RW uint32_t EXTMEM4_CTRL;                                                 /**< : 0x038      Chip Select 4*/
    __RW uint32_t CACHE_HIGH_ADDR;                                              /**< : 0x3C    */
    __RW uint32_t INTMEM_SCR_RNG_ADDR;                                          /**< : 0x40       1*/
    __RW uint32_t INTMEM_SCR_PRD_SCAN;                                          /**< : 0x44       1*/
    __RW uint32_t INTMEM_SCR_PRD_STOP;                                          /**< : 0x48      1*/
    __RW uint32_t INTMEM2_SCR_RNG_ADDR;                                         /**< : 0x4       2.*/
    __RW uint32_t INTMEM2_SCR_PRD_SCAN;                                         /**< : 0x50       2*/
    __RW uint32_t INTMEM2_SCR_PRD_STOP;                                         /**< : 0x54      2*/
    __RW uint32_t INTMEMS_SCR_MAIN;                                             /**< : 0x58   */
    __RW uint32_t CACHE_CRC_ERROR;                                              /**< : 0x5  crc   */  
    __RW uint32_t EDAC_INTMEM_SCR_CERR;                                         /**< : 0x60   ,     1*/
    __RW uint32_t EDAC_INTMEM_SCR_FERR;                                         /**< : 0x64   ,     1*/
    __RW uint32_t EDAC_INTMEM2_SCR_CERR;                                        /**< : 0x68   ,     2*/  
    __RW uint32_t EDAC_INTMEM2_SCR_FERR;                                        /**< : 0x6   ,     2*/ 
    __RW uint32_t DMA_INTR_FLAGS;                                               /**< : 0x70   DMA*/ 
    __RW uint32_t ALT_FUNCTION_CTRL[9];                                         /**< : 0x74-0x94       GPIO*/ 
    __RW uint32_t RESERVED_1[2];                                                /**<  */
    __RW uint32_t CACHE_HIGH_ADDR_CS2;                                          /**< : 0xA0        Chip Select 2*/ 
    __RW uint32_t CACHE_HIGH_ADDR_CS3;                                          /**< : 0x4        Chip Select 3*/
    __RW uint32_t CACHE_HIGH_ADDR_CS4;                                          /**< : 0x8        Chip Select 4*/
    __RW uint32_t ALIAS_CTRL;                                                   /**< : 0x      0x0000_0000-0x00FF_FFFF*/
    __RO uint32_t SCRUBBER_FERR_ADDR;                                           /**< : 0x0   FERR  */
    __RO uint32_t COMMON_FERR_ADDR;                                             /**< : 0x4   FERR       .*/
    __RW uint32_t EDAC_REACTION_CTRL;                                           /**< : 0x8     EDAC  CRC*/
    __RW uint32_t GLOBAL_RESET;                                                 /**< : 0xC   ,   (W1O)*/
    __RW uint32_t RES_P_SYNC;                                                   /**< */
    __RW uint32_t CACHE_MAIN;                                                   /**< : 0xC4  /    */  
}CMN_REG_T;

/**
 * @brief    EXTMEM_CTRL
 */
#define EXT_MEM_READ_CYCLES(x)                  ((uint32_t)(((x) & 0x7) << 0))  /**<   ,          */ 
#define EXT_MEM_WRITE_CYCLES(x)                 ((uint32_t)(((x) & 0x7) << 3))  /**<   ,          */ 
#define EXT_MEM_TURN_CYCLES(x)                  ((uint32_t)(((x) & 0x7) << 6))  /**<    ,     ,   */ 
#define EXT_MEM_SIZE                            ((uint32_t)(1 << 16))           /**<       Chip Select 1.  0  8-  , 
                                                                                     1  16-  .  ,    SIZE_FORCE   1*/
#define EXT_MEM_EDAC                            ((uint32_t)(1 << 24))           /**<        Chip Select 1.
                                                                                     0      , 
                                                                                1     .  ,    EDAC_FORCE   1*/
#define EXT_MEM_SIZE_FORCE                      ((uint32_t)(1 << 30))           /**<     1,       Chip Select 1   Size, 
                                                                                         EXTMEM_CFGSIZE*/
#define EXT_MEM_EDAC_FORCE                      ((uint32_t)(1 << 31))           /**<     1,      Chip Select 1   EDAC, 
                                                                                         EXTMEM_EDAC_EN*/

/**
 * @brief    EDAC_CTRL
 */
#define EDAC_En_1                               ((uint32_t)(1 << 0))            /**<             1.*/
#define EDAC_En_2                               ((uint32_t)(1 << 1))            /**<             2 */
#define EDAC_WR_DIS_1                           ((uint32_t)(1 << 2))            /**<    EDAC    1 */
#define EDAC_WR_DIS_2                           ((uint32_t)(1 << 3))            /**<    EDAC    2 */

/**
 * @brief    INTMEM_CERR_CNT
 */
#define INTMEM_CERR_CNT(x)                   ((uint32_t)((x&0xFFFFFFFF) << 0))  /**<        1.*/

/**
 * @brief    INTMEM_FERR_CNT
 */
#define INTMEM_FERR_CNT(x)                   ((uint32_t)((x&0xFFFFFFFF) << 0))  /**<       1.*/

/**
 * @brief    EXTMEM_CERR_CNT
 */
#define EXTMEM_CERR_CNT(x)                   ((uint32_t)((x&0xFFFFFFFF) << 0))  /**<      */

/**
 * @brief    EXTMEM_FERR_CNT
 */
#define EXTMEM_FERR_CNT(x)                   ((uint32_t)((x&0xFFFFFFFF) << 0))  /**<     */

/**
 * @brief    SPACEWIRE_CLK_CTRL
 */
#define SPACEWIRE_CLK_CTRL(x)                      ((uint32_t)(x << 16))           /**<       Spacewire.
                                                                                    0       SW_CLK (PORTH[4]).
                                                                                    1       
                                                                                */
/**
 * @brief    INTMEM2_CERR_CNT
 */
#define NTMEM2_CERR_CNT(x)                   ((uint32_t)((x&0xFFFFFFFF) << 0))  /**<        2*/

/**
 * @brief    INTMEM2_FERR_CNT
 */
#define INTMEM2_FERR_CNT(x)                   ((uint32_t)((x&0xFFFFFFFF) << 0)) /**<       2.*/

/**
 * @brief    PWR_CTRL_CLK
 *     PWR_CTRL_CLK  PWR_CTRL_RST,
 *     :
 *0  SPI_1	13  Telecommand Decoder (TCRX)
 *1  SPI_2	14  FPU
 *2  UART_1	
 *3  UART_2	
 *4  UART_3	
 *5  UART_4	
 *6  Spacewire_1	
 *7  Spacewire_2	20  CAN_1
 *8  MKPD1	21  CAN_2
 *9  MKPD2	22  UART_5
 *10  MKPD3	23  UART_6
 *11  MKPD4	24  I2C
 *12  Telemetry Encoder (TMTX)	25-31   
 *
 */
#define PWR_CTRL_CLK(x)                   ((uint32_t)((x&0xFFFFFFFF) << 0))     /**<      . (1  ).*/

/**
 * @brief    PWR_CTRL_RST
 *     PWR_CTRL_CLK  PWR_CTRL_RST,      :
 *0  SPI_1	13  Telecommand Decoder (TCRX)
 *1  SPI_2	14  FPU
 *2  UART_1	
 *3  UART_2	
 *4  UART_3	
 *5  UART_4	
 *6  Spacewire_1	
 *7  Spacewire_2	20  CAN_1
 *8  MKPD1	21  CAN_2
 *9  MKPD2	22  UART_5
 *10  MKPD3	23  UART_6
 *11  MKPD4	24  I2C
 *12  Telemetry Encoder (TMTX)	25-31   
 *
 */
#define PWR_CTRL_RST(x)                   ((uint32_t)((x&0xFFFFFFFF) << 0))     /**<         . (1  )*/

/**
 * @brief    EXTMEM_CTRL(Chip Select 2-4) 
 */
#define EXT_MEM_READ_CYCLES_2_4(x)              ((uint32_t)(((x) & 0x7) << 0))  /**<   ,         . 
                                                                                     0  1 ,  1 - 2 ,...  7 - 8  */ 
#define EXT_MEM_WRITE_CYCLES_2_4(x)             ((uint32_t)(((x) & 0x7) << 3))  /**<   ,         . 
                                                                                     0  1 ,  1 - 2 ,...  7 - 8 */ 
#define EXT_MEM_TURN_CYCLES_2_4(x)              ((uint32_t)(((x) & 0x7) << 6))  /**< Turn_cycles 	  Chip Select ( CEN)    ,          . 
                                                                                      0   ,  1 - 1  ,...  7 - 7   */ 
#define EXT_MEM_SIZE_2_4                        ((uint32_t)(1 << 16))           /**< 0 - 8- ; 1 - 16- */
#define EXT_MEM_EDAC_2_4                        ((uint32_t)(1 << 24))           /**<    0 - ; 1  */


/**
 * @brief    CACHE_HIGH_ADDR
 */
#define CACHE_HIGH_ADDR(x)                   ((uint32_t)((x&0x7FFFFF) << 2))   /**<     ,   .   , 
                                                                                         0     . 
                                                                                         ,    , 
                                                                                           */

/**
 * @brief    CACHE_HIGH_ADDR_CS2
 */
#define CACHE_HIGH_ADDR_CS2(x)                ((uint32_t)((x&0x7FFFFF) << 2))   /**<     ,   .   , 
                                                                                         0     . 
                                                                                         ,    , 
                                                                                           */

/**
 * @brief    CACHE_HIGH_ADDR_CS3
 */
#define CACHE_HIGH_ADDR_CS3(x)                ((uint32_t)((x&0x7FFFFF) << 2))   /**<     ,   .   , 
                                                                                         0     . 
                                                                                         ,    , 
                                                                                           */

/**
 * @brief    CACHE_HIGH_ADDR_CS4
 */
#define CACHE_HIGH_ADDR_CS4(x)                ((uint32_t)((x&0x7FFFFF) << 3))   /**<     ,   .   , 
                                                                                         0     . 
                                                                                         ,    , 
                                                                                           */

/**
 * @brief    INTMEM_SCR_RNG_ADDR
 */
#define INTMEM_SCR_RNG_ADDR(x)                ((uint32_t)((x&0x1FFF) << 3))     /**<        */

/**
 * @brief    INTMEM2_SCR_RNG_ADDR
 */
#define INTMEM2_SCR_RNG_ADDR(x)                ((uint32_t)((x&0x1FFF) << 3))    /**<        */

/**
 * @brief    INTMEM_SCR_PRD_SCAN
 */
#define INTMEM_SCR_PRD_SCAN(x)             ((uint32_t)((x&0xFFFFFFFF) << 0))    /**<       .0 -  .*/

/**
 * @brief    INTMEM2_SCR_PRD_SCAN
 */
#define INTMEM2_SCR_PRD_SCAN(x)            ((uint32_t)((x&0xFFFFFFFF) << 0))    /**<       .0 -  .*/

/**
 * @brief    INTMEM_SCR_PRD_STOP
 */
#define INTMEM_SCR_PRD_STOP(x)            ((uint32_t)((x&0xFFFFFFFF) << 0))     /**<     ,       
                                                                                       .0 -  .*/
/**
 * @brief    INTMEM2_SCR_PRD_STOP
 */
#define INTMEM2_SCR_PRD_STOP(x)            ((uint32_t)((x&0xFFFFFFFF) << 0))    /**<     ,       
                                                                                       .0 -  .*/
/**
 * @brief    INTMEMS_SCR_MAIN
 */
#define SCRUB_BLK1_EN                          ((uint32_t)((1<< 0))             /**< RW*/
#define SCRUB_BLK2_EN                          ((uint32_t)((1<< 1))             /**< 1 -      */
#define SCRUB_BLK1_READY                       ((uint32_t)((1<< 2))             /**< 1 -         */
#define SCRUB_BLK2_READY                       ((uint32_t)((1<< 2))             /**< 1 -         */
#define SCRUB_BLK1_CS_READY(x)                 ((uint32_t)((x&0xF) << 4))       /**<       Chip Select (  )*/
#define SCRUB_BLK2_CS_READY(x)                 ((uint32_t)((x&0xF) << 8))       /**<       Chip Select (  )*/

/**
 * @brief    DMA_INTR_FLAGS
 *
 * [15:0]   DMA,   1.
 * [0]   SSP1 TX 
 * [1]   SSP1 RX
 * [2]   SSP2 TX 
 * [3]   SSP2 RX
 * [4]   UART1 TX
 * [5]   UART1 RX
 * [6]   UART2 TX
 * [7]   UART2 RX
 * [8]   UART3 TX
 * [9]   UART3 RX
 * [10]   UART4 TX
 * [11]   UART4 RX
 * [12]   UART5 TX
 * [13]   UART5 RX
 * [14]   UART6 TX
 * [15]   UART6 RX
 */
#define DMA_INTR_FLAGS(x)                  ((uint32_t)(((x) & x&0xFF) << 0))    /**<  */ 

/**
 * @brief    ALT_FUNC_CONTROL
 */
#define BIT_FUNC(x)                      ((uint32_t)((x&0xFFFFFFFF) << 0))      /**<       .0 -  .*/

/**
 * @brief    ALIAS_CTRL
 */
#define INTMEM_ALIAS(x)                 ((uint32_t)((x&0x3) << 0))              /**<    :
                                                                                    0   ;
                                                                                    1     1 (0x2000_0000  0x2000_FFFF); 
                                                                                    2     2 (0x2001_0000  0x2001_FFFF); 
                                                                                    3      (0x2000_0000  0x2001_FFFF). 
                                                                                    .*/
#define EXTMEM_ALIAS(x)                 ((uint32_t)((x&0x3) << 2))              /**<         INTMEM_ALIAS=0:
                                                                                    0      Chip Select 1 
                                                                                    (0x0800_0000  0x08FF_FFFF);
                                                                                    1      Chip Select 2 
                                                                                    (0x0900_0000  0x09FF_FFFF);
                                                                                    2      Chip Select 3 
                                                                                    (0x0A00_0000  0x0AFF_FFFF);
                                                                                    3      Chip Select 4 
                                                                                    (0x0B00_0000  0x0BFF_FFFF).
                                                                                    */      
/**
 * @brief    EDAC_REACTION_CTRL
 *
 *   :
 *0   ;
 *1  ;
 *2  ;
 *3   ,  ;
 *4   ,  ;
 *6-7   .
 *
 */
#define EXTMEM_CERR(x)                 ((uint32_t)((x&0xF) << 0)) 
#define EXTMEM_FERR(x)                 ((uint32_t)((x&0xF) << 3)) 
#define INTMEM_CERR(x)                 ((uint32_t)((x&0xF) << 6)) 
#define INTMEM_FERR(x)                 ((uint32_t)((x&0xF) << 9)) 
#define INTMEM_SCR_CERR(x)             ((uint32_t)((x&0xF) << 12)) 
#define INTMEM_SCR_FERR(x)             ((uint32_t)((x&0xF) << 15)) 
#define CACHE_CRC_ERR(x)               ((uint32_t)((x&0xF) << 18)) 
                                                                                   

/**
 * @brief    CACHE_MAIN
 */
#define CACHE_ENABLE(x)                       ((uint32_t)(x<< 0))                  /**< 1 -   , ; 0 - */
#define CACHE_READY(x)                        ((uint32_t)(x<< 1))                  /**< 1 - , ; 0 -  */

/**
 * @brief	 
 * @param	IRQn	:  
 */
void NVIC_EnableIRQ(IRQn_T IRQn,uint32_t vec);


#define NVIC_En_IRQ(X)          *((volatile unsigned int *)(0xE000E100 + (X>>5)*4)) = (1<<(X%32)); ///< X -  
#define NVIC_Dis_IRQ(X)         *((volatile unsigned int *)(0xE000E180 + (X>>5)*4)) = (1<<(X%32)); ///< X -  


/**
 *\brief        
 */
typedef struct 
{
    __RW uint32_t ISER[8];                                                      /**< : 0x000    */         
    uint32_t RESERVED0[24];                                          
    __RW uint32_t ICER[8];                                                      /**< : 0x080    */         
    uint32_t RESERVED1[24];                                          
    __RW uint32_t ISPR[8];                                                      /**< : 0x100     */          
    uint32_t RESERVED2[24];                                                                  
    __RW uint32_t ICPR[8];                                                      /**< : 0x180     */        
    uint32_t RESERVED4[88];                                                                  
    __RW uint8_t  IP[240];                                                      /**< : 0x300     */ 
}NVIC_T;

/** \brief        
 */
typedef struct 
{
    __RW uint32_t CTRL;                                                         /**< : 0x000     */
    __RW uint32_t LOAD;                                                         /**< : 0x004     */
    __RW uint32_t VAL;                                                          /**< : 0x008    */
    __RO  uint32_t CALIB;                                                       /**< : 0x00C   */
} SYSTICK_T;  

/** \brief        
 */
typedef struct 
{
    __RO  uint32_t CPUID;                                                       /**< : 0x000  CPUID */
    __RW uint32_t ICSR;                                                         /**< : 0x004      */
    __RW uint32_t AIRCR;                                                        /**< : 0x00       */
    __RW uint32_t SCR;                                                          /**< : 0x010    */
    __RO  uint32_t CCR;                                                         /**< : 0x014     */
    __RW uint32_t SHPR2;                                                        /**< : 0x01  2   */
    __RW uint32_t SHPR3;                                                        /**< : 0x020  3   */
} SCB_T;                                                                    



/** \brief      PWR_CLK_CTRL  PWR_RST_CTRL
 *     PWR_CTRL_CLK  PWR_CTRL_RST,
 *     :
 *0  SPI_1	13  Telecommand Decoder (TCRX)
 *1  SPI_2	14  FPU
 *2  UART_1	
 *3  UART_2	
 *4  UART_3	
 *5  UART_4	
 *6  Spacewire_1	
 *7  Spacewire_2	20  CAN_1
 *8  MKPD1	21  CAN_2
 *9  MKPD2	22  UART_5
 *10  MKPD3	23  UART_6
 *11  MKPD4	24  I2C
 *12  Telemetry Encoder (TMTX)	25-31   
 */
typedef enum 
{
    SPI_1_CTRL_NUM                   = 0,                                        /**<      */ 
    SPI_2_CTRL_NUM                   = 1,                                        /**<     */ 
    UART_1_CTRL_NUM                  = 2,                                        /**<   UART 1 */
    UART_2_CTRL_NUM                  = 3,                                        /**<   UART 2 */
    UART_3_CTRL_NUM                  = 4,                                        /**<   UART 3 */
    UART_4_CTRL_NUM                  = 5,                                        /**<   UART 4 */
    Spacewire_1_CTRL_NUM             = 6,                                        /**<   SpaceWire 1 */  
    Spacewire_2_CTRL_NUM             = 7,                                        /**<   SpaceWire 2 */  
    MKPD1_CTRL_NUM                  = 8,                                        /**<   Manhester 0 */ 
    MKPD2_CTRL_NUM                 = 9,                                         /**<   Manhester 1 */ 
    MKPD3_CTRL_NUM                  = 10,                                       /**<   Manhester 2 */ 
    MKPD4_CTRL_NUM                 = 11,                                        /**<   Manhester 3 */
    TMTX_CTRL_NUM                = 12,                                          /**<   TMTX */
    TCRX_CTRL_NUM                = 13,                                          /**<   TCRX */
    FPU_CTRL_NUM                 = 14,                                          /**<   FPU */
    RES_P1_CTRL_NUM               =15,                                          /**<  */
    RES_P2_CTRL_NUM               =16,                                          /**<  */
    RES_P3_CTRL_NUM               =17,                                          /**<  */
    RES_P4_CTRL_NUM               =18,                                          /**<  */
    RES_CTRL_NUM                 =19,                                           /**<  */
    CAN_1_CTRL_NUM               =20,                                           /**<   CAN1 */
    CAN_2_CTRL_NUM               =21,                                           /**<   CAN2 */
    UART_5_CTRL_NUM              = 22,                                          /**<   UART 5 */
    UART_6_CTRL_NUM              = 23,                                          /**<   UART 6 */
    I2C_CTRL_NUM                 =24
} CTRL_T;

/**
 * @brief	 
 * @param	IRQn	:  
 */
void NVIC_EnableIRQ(IRQn_T IRQn,uint32_t vec);
/**
 * @brief	 
 * @param	vec	:   
 */
void NVIC_DisableIRQ(uint32_t vec);

/**
 * @brief    SYSTICK CTRL
 */
#define SYST_CTRL_EN                         ((uint32_t)(1 << 0))               /**<   */ 
#define SYST_CTRL_INT_EN                     ((uint32_t)(1 << 1))               /**<    */ 
#define SYST_CTRL_CLKSRC_REF                 ((uint32_t)(0 << 2))               /**<       */ 
#define SYST_CTRL_CLKSRC_CPU                 ((uint32_t)(1 << 2))               /**<       */ 
#define SYST_CTRL_COUNTFL                    ((uint32_t)(1 << 16))              /**<     ,    1,    0       */ 

/**
 * @brief    SYSTICK LOAD
 */
#define SYST_LOAD(x)                         ((uint32_t)((x) & 0xFFFFFF))       /**<     VAL,      0*/ 

/**
 * @brief    SYSTICK VAL
 */
#define SYST_VAL(x)                          ((uint32_t)((x) & 0xFFFFFF))       /**<   */ 

/**
 * @brief    SYSTICK VAL
 */
#define SYST_CALIB_TENMS(x)                  ((uint32_t)((x) & 0xFFFFFF))       /**<   10 (100Hz),      */ 
#define SYST_CALIB_SKEW                      ((uint32_t)(1 << 30))              /**<     , ,   SYST_CALIB_TENMS */ 
#define SYST_CALIB_NOREF                     ((uint32_t)(1 << 31))              /**<     , ,        */ 

/**
 * @brief    SCB CPUID
 */
#define SCB_CPUID_IMP(x)                     ((uint32_t)(((x)  >> 0) & 0xF))    /**<   */ 
#define SCB_CPUID_VAR(x)                     ((uint32_t)(((x) >> 4) & 0xFFF))   /**<   (r)     rnpn */ 
#define SCB_CPUID_CONST(x)                   ((uint32_t)(((x) >> 16) & 0xF))    /**<   */ 
#define SCB_CPUID_PRTN(x)                    ((uint32_t)(((x) >> 20) & 0xF))    /**<   */ 
#define SCB_CPUID_REV(x)                     ((uint32_t)(((x) >> 24) & 0xFF))   /**<   (p)     rnpn */ 

/**
 * @brief    SCB ICSR
 */
#define SCB_ICSR_VECTACTIVE(x)               ((uint32_t)(((x) >> 0) & x1F))     /**<    */ 
#define SCB_ICSR_VECTPENDING(x)              ((uint32_t)(((x) >> 12) & 0x1F))   /**<        */ 
#define SCB_ICSR_ISRPENDING                  ((uint32_t)(1 << 22))              /**<       */ 
#define SCB_ICSR_PENDSTCLR                   ((uint32_t)(1 << 25))              /**<    SYSTICK */ 
#define SCB_ICSR_PENDSTSET                   ((uint32_t)(1 << 26))              /**<    SYSTICK */ 
#define SCB_ICSR_PENDSVCLR                   ((uint32_t)(1 << 27))              /**<   PendSV */ 
#define SCB_ICSR_PENDSVSET                   ((uint32_t)(1 << 28))              /**<   PendSV */ 
#define SCB_ICSR_NMIPENDSET                  ((uint32_t)(1 << 31))              /**<   NMI */ 

/**
 * @brief    SCB AIRCR
 */
#define SCB_AIRCR_VECTKEY                    ((uint32_t)((0x05FA) << 16))       /**<  ,     */ 
#define SCB_AIRCR_ENDIANESS                  ((uint32_t)(1 << 15))              /**<     */ 
#define SCB_AIRCR_SYSRESETREQ                ((uint32_t)(1 << 2))               /**<    */ 
#define SCB_AIRCR_VECTCLRACTIVE              ((uint32_t)(0 << 1))               /**<      */ 

/**
 * @brief    SCB SCR
 */
#define SCB_SCR_SEVONPEND                    ((uint32_t)(1 << 4))               /**<          */ 
#define SCB_SCR_SLEEPDEEP                    ((uint32_t)(1 << 2))               /**<        */ 
#define SCB_SCR_SLEEPONEXIT                  ((uint32_t)(1 << 1))               /**<          */ 

/**
 * @brief    SCB SHPR2
 */
#define SCB_SHPR2_PRI_11(x)                  ((uint32_t)(((x) & 0x3) << 30))    /**<    SVCall */ 

/**
 * @brief    SCB SHPR3
 */
#define SCB_SHPR3_PRI_14(x)                  ((uint32_t)(((x) & 0x3) << 22))    /**<    PendSV */ 
#define SCB_SHPR3_PRI_15(x)                  ((uint32_t)(((x) & 0x3) << 30))    /**<    SysTick */ 



              

/**
 * @brief    ALTFUNC_CTRL
 */
#define PORT_A_ALTFUNC                          0                               /**<   PORT_A */
#define PORT_B_ALTFUNC                          1                               /**<   PORT_B */
#define PORT_C_ALTFUNC                          2                               /**<   PORT_C */
#define PORT_D_ALTFUNC                          3                               /**<   PORT_D */
#define PORT_E_ALTFUNC                          4                               /**<   PORT_E */
#define PORT_F_ALTFUNC                          5                               /**<   PORT_F */
#define PORT_G_ALTFUNC                          6                               /**<   PORT_G */
#define PORT_H_ALTFUNC                          7                               /**<   PORT_H */
#define PORT_I_ALTFUNC                          8                               /**<   PORT_I */

#define ALTFUNC_CTRL_EXTMEM_EDAC_CERR_EN        ((uint32_t)(1 << 0))            /**<   MKPD4_TX/GPIOF[14] */

#define ALTFUNC_CTRL_EXTMEM_EDAC_FERR_EN        ((uint32_t)(1 << 1))            /**<   MKPD4_nTX/GPIOF[15] */

#define ALTFUNC_CTRL_INTMEM_EDAC_CERR_EN        ((uint32_t)(1 << 2))            /**<   MKPD4_TXEN_A/GPIOG[0] */

#define ALTFUNC_CTRL_INTMEM_EDAC_CERR_2_EN      ((uint32_t)(1 << 3))            /**<   SW_SOUT[1]/GPIOG[10] */

#define ALTFUNC_CTRL_INTMEM_EDAC_FERR_2_EN      ((uint32_t)(1 << 4))            /**<   SW_DOUT[1]/GPIOG[11] */

#define ALTFUNC_CTRL_BITMASK                    ((uint32_t)(0x1F << 0))         /**<   */

#define SET_ALTFUNC(PORT, BIT, FUNC) CMN_REG->ALT_FUNCTION_CTRL[PORT] = (CMN_REG->ALT_FUNCTION_CTRL[PORT] & (~(3 << (2 * BIT)))) | (FUNC << (2 * BIT))  /**<    */


/**
 * @brief    MKPD_CLK_CTRL
 */
#define MKPD_CLK_CTRL_TX(x)                     ((uint32_t)((x) & 0xFF) << 0)   /**<           */ 

#define MKPD_CLK_CTRL_RX(x)                     ((uint32_t)((x) & 0xFF) << 8)   /**<     ,      */ 

#define MKPD_CLK_SPW_CLK_EN                     ((uint32_t)(1 << 16))           /**<   SW_DOUT[1]/GPIOG[11] */

/**
 * @brief	    
 * @param	CTRLn	:  
 */
void PWR_CLK_Enable(CTRL_T CTRLn);
/**
 * @brief	    
 * @param	CTRLn	:  
 */
void PWR_CLK_Disable(CTRL_T CTRLn);
/**
 * @brief	     
 * @param	CTRLn	:  
 */
void PWR_RST_Disable(CTRL_T CTRLn);

/**
 * @brief	    
 * @param	CTRLn	:  
 */
void PWR_RST_Enable(CTRL_T CTRLn);

/**
 * @brief	 
 */
void SystemInit2(void);

#endif

