/*!\file
 * \brief         SPI.
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#include "spi_test.h"

const uint32_t sys_freq = 80000000;                                             //  
char dbg_buffer[256];                                                           //   


#define EXT
#define SPI_BUF_SIZE 64   
#define SPI_DATA_BIT_NUM(databits)          (databits+1)
#define SPI_DATA_BYTES(databits)            (((databits) > SPI_BITS_8) ? 2:1)
#define SPI_LO_BYTE_MSK(databits)           ((SPI_DATA_BYTES(databits) > 1) ? 0xFF:(0xFF>>(8-SPI_DATA_BIT_NUM(databits))))
#define SPI_HI_BYTE_MSK(databits)           ((SPI_DATA_BYTES(databits) > 1) ? (0xFF>>(16-SPI_DATA_BIT_NUM(databits))):0)
   

#pragma data_alignment = 0x400
#pragma location=".noinit"
DMA_CTRL_T DMA_SPI_CTRL;

static uint8_t dma_tx_done;
static uint8_t dma_rx_done;
static uint8_t int_done;
static uint16_t Master_Tx_Buf[SPI_BUF_SIZE];
static uint16_t Slave_Tx_Buf[SPI_BUF_SIZE];
static uint16_t Master_Rx_Buf[SPI_BUF_SIZE];
static uint16_t Slave_Rx_Buf[SPI_BUF_SIZE];
static SPI_CONFIG_T ssp_config;
static SPI_DATA_SETUP_T master_setup;
static SPI_DATA_SETUP_T slave_setup;

/**
 * @brief    SPI      
 */
void Init_ports_SPI()
{  
    CMN_REG->ALT_FUNCTION_CTRL[3] =  0;                                         //  GPIO_E 
    GPIO_D->ALTFUNCSET =(GPIO_D->ALTFUNCSET&0)|(1<<7)|(1<<8)|(1<<9)|(1<<10)     
                               |(1<<11)|(1<<12)|(1<<13)|(1<<14);  
}

/**
 * @brief     
 */
static uint8_t Buffer_Verify(uint8_t bits, uint16_t * src_addr, uint16_t * dst_addr)
{
    uint16_t i;
    for ( i = 0; i < SPI_BUF_SIZE; i++ )
    {
	if (((*src_addr) & SPI_LO_BYTE_MSK(bits)) !=
            ((*dst_addr) & SPI_LO_BYTE_MSK(bits))) 
        {
            return 1;
	}
	src_addr++;
	dst_addr++;

	if (SPI_DATA_BYTES(bits) == 2)
        {
            if (((*src_addr) & SPI_HI_BYTE_MSK(bits)) !=
			  ((*dst_addr) & SPI_HI_BYTE_MSK(bits))) 
            {
		return 1;
            }
            src_addr++;
            dst_addr++;
            i++;
	}
    }
    return 0;
}

/**
 * @brief	   SPI1
 */
void SPI_1_IRQ(void)
{

  
    for (int i = 0; i < SPI_BUF_SIZE; i++) 
    {
        SPI_SendFrame(SPI1, Master_Tx_Buf[i]);
        while (SPI_GetStatus(SPI1, SPI_SR_BSY) == SET) {}
        Slave_Rx_Buf[i] = SPI_ReceiveFrame(SPI2);
    }
    SPI_TXInt_Disable(SPI1);
    SPI_RXInt_Enable(SPI2);	
    NVIC_DisableIRQ(6); 
    int_done = 1;
}

/**
 * @brief	   SPI2
 */
void SPI_2_IRQ(void)
{

    for (int i = 0; i < SPI_BUF_SIZE; i++) 
    {
        while (!(SPI_GetStatus(SPI2, SPI_SR_RFF) == SET)) {}
        Slave_Rx_Buf[i] = SPI_ReceiveFrame(SPI2);
    }
    int_done = 1;
    NVIC_DisableIRQ(7);
}

/**
 * @brief	   DMA SPI1
 */
void DMA_DONE_SPI_1_IRQ(void)
{
    dma_tx_done = 1;
    SPI1->DMACR &= (~SPI_DMACR_TXDMAE) & SPI_DMACR_BITMASK;
    NVIC_DisableIRQ(26);
}
  
/**
 * @brief	   DMA SPI2
 */
void DMA_DONE_SPI_2_IRQ(void)
{
    dma_rx_done = 1;
    SPI2->DMACR &= (~SPI_DMACR_RXDMAE) & SPI_DMACR_BITMASK;
    NVIC_DisableIRQ(27);
} 
void start_init()
{
        uint16_t i = 0;     
    ssp_config.bits = SPI_BITS_16;
    ssp_config.clockMode = SPI_CLOCK_MODE1;
    ssp_config.frameFormat = SPI_FRF_SPI;
        
    SPI_Init(SPI1);        
    SPI_Set_Mode(SPI1, SPI_MODE_MS_MASTER);
    SPI_SetFormat(SPI1, ssp_config.bits, ssp_config.clockMode, ssp_config.frameFormat);
    SPI_SetBitRate(SPI1, 8000000);
        
    SPI_Init(SPI2);
    SPI_Set_Mode(SPI2, SPI_MODE_MS_SLAVE);
    SPI_SetFormat(SPI2, ssp_config.bits, ssp_config.clockMode, ssp_config.frameFormat);
    SPI_SetBitRate(SPI2, 8000000);

    SPI_Enable(SPI1);
    SPI_Enable(SPI2);
        
    for (i = 0; i < SPI_BUF_SIZE; i++)
    {
        Master_Tx_Buf[i] = i;
        Master_Rx_Buf[i] = 0xAAAA;
        Slave_Tx_Buf[i] = i;
        Slave_Rx_Buf[i] = 0xAAAA;
    }
        
    master_setup.length = SPI_BUF_SIZE << (SPI_DATA_BYTES(ssp_config.bits) - 1);
	master_setup.tx_data = Master_Tx_Buf;
	master_setup.rx_data = Master_Rx_Buf;  
        master_setup.rx_cnt = master_setup.tx_cnt = 0;
        
        slave_setup.length = SPI_BUF_SIZE << (SPI_DATA_BYTES(ssp_config.bits) - 1);
	slave_setup.tx_data = Slave_Tx_Buf;
	slave_setup.rx_data = Slave_Rx_Buf;  
        slave_setup.rx_cnt = master_setup.tx_cnt = 0;
}
/**
 * @brief	   SPI
 */
uint32_t SPI_Test(void)
{
    start_init();   
    Init_ports_SPI();                                                           //   GPIO   SPI 
        
    PRINT(" SPI");
        
/*****************************************************************************
 *  
 ****************************************************************************/        
        
    PRINTSTREAM("    ");       

    for (int i = 0; i < SPI_BUF_SIZE; i++) 
    {
        SPI_SendFrame(SPI1, Master_Tx_Buf[i]);
        while (SPI_GetStatus(SPI1, SPI_SR_BSY) == SET) {}
        Slave_Rx_Buf[i] = SPI_ReceiveFrame(SPI2);
    }
      
    if (Buffer_Verify(16, Master_Tx_Buf, Slave_Rx_Buf) == 0) 
    {
        PRINTSTREAM("\r\n");
    }
    else 
    {
        PRINTSTREAM(" \r\n");
        SPI_DeInit(SPI1);
        SPI_DeInit(SPI2);          
        PRINT(" SSP   ");
        return false;
    }
        
    for (int i = 0; i < SPI_BUF_SIZE; i++)
    {
        Master_Tx_Buf[i] = i;
        Master_Rx_Buf[i] = 0xAAAA;
        Slave_Tx_Buf[i] = i;
        Slave_Rx_Buf[i] = 0xAAAA;
    }
    master_setup.rx_cnt = master_setup.tx_cnt = 0;
    slave_setup.rx_cnt = master_setup.tx_cnt = 0;
    DBG_TX_Flush();
        
/*****************************************************************************
 *  c 
 ****************************************************************************/ 
   
        
    PRINTSTREAM("  c   "); 
    start_init();
    int_done = 0;
    SPI_Int_FlushData(SPI1);
    SPI_Int_FlushData(SPI2);      

    SPI_TXInt_Enable(SPI1);
       
    NVIC_EnableIRQ(SPI_1,6); 
        
    while (!int_done) {}
        
    NVIC_DisableIRQ(SPI_1); 
    NVIC_DisableIRQ(SPI_2);
                               
    if (Buffer_Verify(16, Master_Tx_Buf, Slave_Rx_Buf) == 0)
    {
        PRINTSTREAM("\r\n");
    }
    else
    {
        PRINTSTREAM(" \r\n");
        SPI_DeInit(SPI1);
        SPI_DeInit(SPI2);          
        PRINT(" SSP   ");
        return false;
    }

    for (int i = 0; i < SPI_BUF_SIZE; i++)
    {
        Master_Tx_Buf[i] = i;
        Master_Rx_Buf[i] = 0xAAAA;
        Slave_Tx_Buf[i] = i;
        Slave_Rx_Buf[i] = 0xAAAA;
    }
    master_setup.rx_cnt = master_setup.tx_cnt = 0;
    slave_setup.rx_cnt = master_setup.tx_cnt = 0;
    DBG_TX_Flush();

#ifndef EXT
/*****************************************************************************
 *  c DMA
 ****************************************************************************/ 

    start_init();
        
    PRINTSTREAM("  c DMA  "); 
    dma_tx_done = 0;
    dma_rx_done = 0;
       
    DMA->CFG = 1; 
    DMA->CTRL_BASE_PTR =  (uint32_t) &DMA_SPI_CTRL;
       
    DMA->CHNL_ENABLE_SET = 0x0F;
    DMA->CHNL_USEBURST_CLR |= 0x0F;
        
    DMA_SPI_CTRL.prim_chan[0].src_data_end_ptr = (uint32_t)&Master_Tx_Buf[SPI_BUF_SIZE - 1];
    DMA_SPI_CTRL.prim_chan[0].dst_data_end_ptr = SPI1_BASE + 8;
    DMA_SPI_CTRL.prim_chan[0].ctrl = DMA_CCTRL_BASIC | DMA_NEXT_USEBURST_DIS | \
                                     DMA_N_MINUS_1(SPI_BUF_SIZE - 1) | DMA_R_POWER(2) | \
                                     DMA_SRC_PROT_CACHE_DIS | DMA_SRC_PROT_BUFFER_DIS | DMA_SRC_PROT_PRIVILEGE_DIS | \
                                     DMA_DST_PROT_CACHE_DIS | DMA_DST_PROT_BUFFER_DIS | DMA_DST_PROT_PRIVILEGE_DIS | \
                                     DMA_SRC_DATA_SIZE_HALFWORD | DMA_SRC_ADDR_INC_HALFWORD | \
                                     DMA_DST_DATA_SIZE_HALFWORD | DMA_DST_ADDR_INC_NULL; 

    DMA_SPI_CTRL.prim_chan[3].src_data_end_ptr = SPI2_BASE + 8;
    DMA_SPI_CTRL.prim_chan[3].dst_data_end_ptr = (uint32_t)&Slave_Rx_Buf[SPI_BUF_SIZE - 1];
    DMA_SPI_CTRL.prim_chan[3].ctrl = DMA_CCTRL_BASIC | DMA_NEXT_USEBURST_DIS | \
                                     DMA_N_MINUS_1(SPI_BUF_SIZE - 1) | DMA_R_POWER(2) | \
                                     DMA_SRC_PROT_CACHE_DIS | DMA_SRC_PROT_BUFFER_DIS | DMA_SRC_PROT_PRIVILEGE_DIS | \
                                     DMA_DST_PROT_CACHE_DIS | DMA_DST_PROT_BUFFER_DIS | DMA_DST_PROT_PRIVILEGE_DIS | \
                                     DMA_SRC_DATA_SIZE_HALFWORD | DMA_SRC_ADDR_INC_NULL | \
                                     DMA_DST_DATA_SIZE_HALFWORD | DMA_DST_ADDR_INC_HALFWORD; 

    NVIC_EnableIRQ(DMA_SPI_1_TX,8);
    NVIC_EnableIRQ(DMA_SPI_1_RX,8);
    NVIC_EnableIRQ(DMA_SPI_2_TX,9);
    NVIC_EnableIRQ(DMA_SPI_2_RX,9);
        
    SPI1->DMACR |= SPI_DMACR_TXDMAE;
    SPI2->DMACR |= SPI_DMACR_RXDMAE;
        
    while ((!(dma_tx_done)) || (!(dma_rx_done))) {}
        
    if (Buffer_Verify(16, Master_Tx_Buf, Slave_Rx_Buf) == 0) 
    {
        PRINTSTREAM("\r\n");
    }
    else
    {
        PRINTSTREAM(" \r\n");
        SPI_DeInit(SPI1);
        SPI_DeInit(SPI2);                    
        PRINT(" SPI   ");
        return false;
    }
#endif
        SPI_DeInit(SPI1);
        SPI_DeInit(SPI2);                  
        PRINT(" SPI  ");
        return true;  
}


