
/*!\file
 * \brief         TCRX.
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */


#include "tcrx_test.h"


#define TEST_TCRX
#define CCSDS_TERMINAL

#ifdef TEST_TCRX

#define RANDOMIZER_SEL_TCRX  0x01
#define BCH_SEL         0x02
#define CONVOLUTION_SEL 0x04
#define DSCR_COUNT      16
#define DSCR_SIZE       1024                                                    //size of each descriptor in bytes
#define USED_COUNT      DSCR_COUNT                                              //count of descriptors, used in test
const uint32_t sys_freq = 80000000;                                             //  
char dbg_buffer[256];                                                           //   

//#define DEBUG_TCRX

unsigned int rx_buffer [(DSCR_SIZE/4)*USED_COUNT];
unsigned int TCRX_cur_dscr; //index of dscr, which expected to be filled by the decoder currently
unsigned int nextExpectedSize; //size of next packet to be received. Should be 7*N, but not more than 1024

//variables for debug
volatile unsigned int TCRX_irq_counter;
unsigned int Break_test;
volatile TCRX_T* tcrx = (TCRX_T*)TCRX;

unsigned int TCRX_receive_complete;
unsigned int TCRX_stage;


unsigned int TCRX_errors;
#ifdef CCSDS_TERMINAL
#       define TEST_TIMEOUT_INIT        10000000

unsigned int test_timeout;
#endif

void InitDscr(unsigned char index)
{
    TCRX->DSCR_TBL[index].DSCR_PTR = (unsigned int) &rx_buffer[index*DSCR_SIZE/4];
    //if size of descriptor is 1024, we should write 0 to flags_size register (just 10 bits present):
    TCRX->DSCR_TBL[index].DSCR_FLAG_SIZE = (1<<31) | ((DSCR_SIZE<1024)?DSCR_SIZE:0);
}

void InitCoder(unsigned int coder_sel)
{
    TCRX->GLOBAL_ENABLE=0x00;
    TCRX->CODER_SEL = coder_sel;

    TCRX->DSCR_CURRENT = 0;
    TCRX_cur_dscr = 0;
    nextExpectedSize = 7;  
    TCRX->GLOBAL_ENABLE=0x01;
}



#ifdef DEBUG_TCRX


#else

void TCRX_IRQ(void)
{             
#ifdef CCSDS_TERMINAL
    test_timeout = TEST_TIMEOUT_INIT;
#endif
  
        //check if decoder deactivated 'present' flag
    if (TCRX->DSCR_TBL[TCRX_cur_dscr].DSCR_FLAG_SIZE >> 31)
    {
        PRINT("\tERROR:'Present' flag in decoder was deactivated\n"); 
        Break_test = 1;
        TCRX_errors++;
        DBG_TX_Flush(); 

        return;
    }
                
        //read all data from descriptor
    unsigned int word;
    unsigned int frame_size = TCRX->DSCR_TBL[TCRX_cur_dscr].DSCR_FLAG_SIZE & 0x3FF;
    if (frame_size == 0)
        frame_size = 1024;
       
        //check frame size
                
    if (frame_size != nextExpectedSize)
    {
        PRINT("\tERROR: TCRX frame size is not correct = %d\n", frame_size);
        PRINT("\tIrq_counter = %d\n", TCRX_irq_counter);
        DBG_TX_Flush(); 
        Break_test = 1;
        TCRX_errors++;
                /////return;
               // ErrorStop(0xE1);
    }
    else
    {
                //PRINT("\tTCRX frame size = %d\n", frame_size);
    }
        
        //TCRX->GLOBAL_ENABLE=0x00;
        //for(int i=0; i<10000; i++);      
        //TCRX->GLOBAL_ENABLE=0x01;
        
    nextExpectedSize += 7;
    if (nextExpectedSize > 1024)
        nextExpectedSize = 7;         
        
        //check data validity
        //unsigned int verified_bytes = 0;
    unsigned int a = TCRX_cur_dscr * DSCR_SIZE/4;
     //in this test decoded bytes will be compared with pattern-counter. the next byte to be decoded, should be equal to value of this variable
    unsigned int nextExpectedWord = 0x03020100; 
        
    for (unsigned int i = 0; i < frame_size/4; i+=1)  
    {                
        word = rx_buffer[a + i];
                                
        if (word != nextExpectedWord)
        {
            PRINT("\tERROR: TCRX decoded data is not correct\n");
            PRINT("\tIrq_counter = %d\n", TCRX_irq_counter);
            PRINT("\t0x%08x\n", word);                
            Break_test = 1;
            TCRX_errors++;
            DBG_TX_Flush(); 
            return;                        
       }
       else
       {
       }
                        
       if (nextExpectedWord == 0xFFFEFDFC)
           nextExpectedWord = 0x03020100;
       else
           nextExpectedWord += 0x04040404;                                
    }     
     
    if (frame_size % 4 != 0)
    {      
        word = rx_buffer[a+frame_size/4];               
        a = ~(0xFFFFFFFF << 8*(frame_size%4));
               
        if ((word ^ nextExpectedWord)&a != 0)
        {
            PRINT("\tERROR: TCRX test - last word is not correct\n");
            PRINT("0x%08x\n", word & a);                        
            Break_test = 1;
            TCRX_errors++;
            DBG_TX_Flush(); 
            return;
        }
    }
                
    InitDscr(TCRX_cur_dscr); //reset descriptor for further use
    TCRX_cur_dscr++;
    TCRX_cur_dscr %= USED_COUNT;
        
    TCRX_irq_counter++;
///TCRX_irq_flag = 1;

    if (nextExpectedSize == 700+7) //subtest ok
    {
        TCRX_stage++;
        TCRX_receive_complete = 1;
        PRINT("\t\tTest progress: ok\n");
        DBG_TX_Flush(); 
    }
}

#endif

void Gpio_TCRX_Init()
{
    CMN_REG->ALT_FUNCTION_CTRL[6] = 0;                                          //  GPIO_G
    CMN_REG->ALT_FUNCTION_CTRL[7] = 0;                                          //  GPIO_H
      
    GPIO_G->ALTFUNCSET = 0;
    GPIO_H->ALTFUNCSET = 0;
   
    GPIO_G->ALTFUNCSET = (1<<15);
    GPIO_H->ALTFUNCSET = (1<<0) | (1<<1) | (1<<2) | (1<<3);
}

/**
 * @brief       TCRX
 */  
void TCRX_Clock_init()
{
    PWR_CLK_Enable(TCRX_CTRL_NUM);                                              //     
    PWR_RST_Disable(TCRX_CTRL_NUM);                                             //     
}

void Test_TCRX()
{
    PRINT("\n********** TCRX test start **********\n");
    DBG_TX_Flush(); 

    Gpio_TCRX_Init();
    TCRX_Clock_init();

    unsigned int errors = 0;
    TCRX_errors = 0;
        
    TCRX_irq_counter = 0;

    //initialize all descriptors
    for (unsigned int i = 0; i < USED_COUNT; i++)
         InitDscr(i);       
        
#warning Please, select the proper coder settings:
      
        for (int i=0;i<10000;i++);
    
    TCRX->START_SEQ = 0xEB90 | (1<<31); //0xAA55;
    TCRX->TAIL_SEQ1 = 0x11223344;
    TCRX->TAIL_SEQ2 = 0x55667788;
       
    TCRX_Enable();                                                              //   ,      
                   
    TCRX_receive_complete = 1;
    TCRX_stage = 0;
    Break_test = 0; 
        
    NVIC_EnableIRQ(TCRX_NUM,19);
    
#ifdef CCSDS_TERMINAL
    test_timeout = TEST_TIMEOUT_INIT;
#endif
        
    for (unsigned int k = 0; k < (DSCR_SIZE/4)*USED_COUNT; k++)
         rx_buffer[k] = 0;
        
#ifdef DEBUG_TCRX
        //manual BCH experiments
    while(1)
    {
        if (TCRX_receive_complete)
        {
            TCRX_receive_complete = 0;
            PRINT("\t1-bit bus, BCH=ON, Randomizer=OFF, Convolution=OFF\n"); 
            DBG_TX_Flush(); 
            InitCoder(BCH_SEL);
        }
                
        if ((TCRX->DSCR_CURRENT & 0xF) >= USED_COUNT)
            TCRX->DSCR_CURRENT &= 0x80000000;
    }
#endif

    while(!Break_test)
    {
        if (TCRX_receive_complete)
        {
            TCRX_receive_complete = 0;
            switch (TCRX_stage)
            {
                case 0: 
                    PRINT("\t\tsubtest#0 1-bit bus, BCH=OFF, Randomizer=OFF, Convolution=OFF\n"); 
                    DBG_TX_Flush(); 
                    InitCoder(0);
#ifdef CCSDS_TERMINAL
                    PRINT("COMMAND=TCRX_START_0\n");
                    DBG_TX_Flush(); 
#endif
                    break;
                case 1:
                    PRINT("\t\tsubtest#1 1-bit bus, BCH=ON, Randomizer=OFF, Convolution=OFF\n"); 
                    DBG_TX_Flush(); 
                    InitCoder(BCH_SEL);
#ifdef CCSDS_TERMINAL
                    PRINT("COMMAND=TCRX_START_1\n");
                    DBG_TX_Flush(); 
#endif
                    break;
                case 2:
                    PRINT("\t\tsubtest#2 1-bit bus, BCH=OFF, Randomizer=ON, Convolution=OFF\n"); 
                    DBG_TX_Flush(); 
                    InitCoder(RANDOMIZER_SEL_TCRX);
#ifdef CCSDS_TERMINAL
                    PRINT("COMMAND=TCRX_START_2\n");
                    DBG_TX_Flush(); 
#endif
                    break;
                case 3:
                    PRINT("\t\tsubtest#3 1-bit bus, BCH=OFF, Randomizer=OFF, Convolution=ON\n"); 
                    DBG_TX_Flush(); 
                    InitCoder(CONVOLUTION_SEL);
#ifdef CCSDS_TERMINAL
                    PRINT("COMMAND=TCRX_START_3\n");
                    DBG_TX_Flush(); 
#endif
                    break;
                case 4:
                    PRINT("\t\tsubtest#4 1-bit bus, BCH=ON, Randomizer=ON, Convolution=ON\n"); 
                    DBG_TX_Flush(); 
                    InitCoder(BCH_SEL | RANDOMIZER_SEL_TCRX | CONVOLUTION_SEL);
#ifdef CCSDS_TERMINAL
                    PRINT("COMMAND=TCRX_START_4\n");
                    DBG_TX_Flush(); 
#endif
                    break;
                                
                case 5:
                    PRINT("\t\tsubtest#5 3-bit bus, BCH=ON, Randomizer=ON, Convolution=ON\n"); 
                    DBG_TX_Flush(); 
                    InitCoder(BCH_SEL | RANDOMIZER_SEL_TCRX | CONVOLUTION_SEL | (1 << 16));
#ifdef CCSDS_TERMINAL
                    PRINT("COMMAND=TCRX_START_5\n");
                    DBG_TX_Flush(); 
#endif
                    break;        
                case 6:
                    PRINT("\t\tTest result - OK\n");                                 
                    Break_test = 1;
                    DBG_TX_Flush(); 
                    break;
           }                        
       }
                          
       if ((TCRX->DSCR_CURRENT & 0xF) >= USED_COUNT)
           TCRX->DSCR_CURRENT &= 0x80000000;
                
       if (TCRX_errors > 0)
       {
           Break_test = 1;
           break;
       }      
                
#ifdef CCSDS_TERMINAL
       if (!--test_timeout)
       {
            PRINT("\tERROR: no activity on TCRX interface. Exit test by timeout\n");
            Break_test = 1;
            TCRX_errors++;
            DBG_TX_Flush(); 
            break;
       }
#endif
    }  
    NVIC_Dis_IRQ(19);  
      
    errors = TCRX_errors;
        
TCRX_test_end:
        
    PRINT("\tTCRX test complete. Errors count = %d\n", errors);
    DBG_TX_Flush(); 
        
}

#endif //TEST_TCRX


