
/*!\file
 * \brief         TMTX.
 *
 * Copyright(C)   ,2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#include "tmtx_test.h"


const uint32_t sys_freq = 80000000;                                             //  
char dbg_buffer[256];                                                           //   
#define TEST_TMTX                                                               //   tmtx
#define CCSDS_TERMINAL                                                          //    !

#ifdef TEST_TMTX

#define DUAL_BASIS              1                                               // 1 -  dual basis   -
#define FRAME_SIZE_NO_CODING    2048                                            

#define NEXT_VAL        c++;            \
                        if (c == 0)                 \
                                c = 1
                                  
#define INIT_FRAME(index)       /*for(unsigned int i=0; i<Frame_size; i++)                        \
                                {                                                               \
                                  frame ## index ## [i] = NEXT_VAL;                             \
                                }                                                               \
                                TMTX->DSCR ## index ## _FLAG_SIZE = ((unsigned int)1<<31) | (Frame_size-1)
*/

unsigned char tmtx_test_failed;
unsigned int Frame_size;
unsigned int FramesTotalCount;                                                  //  
unsigned int FramesSendedCount;                                                 //   
unsigned char frame1 [2048];
unsigned char frame2 [2048];
unsigned int tmtx_cur_dscr;                                                     //  dscr
unsigned char c;                                                                //  
unsigned int Irq_counter;                                                       //   
TMTX_T * tmtx = (TMTX_T *)TMTX;                                                 //  

/**
 * @brief   
 */  
unsigned int GetFrameSize()
{
    switch (TMTX->CODER_SEL & 0x3)          
    {
        case 0:
            return FRAME_SIZE_NO_CODING;
        case TMTX_CODER_SEL_LDPC:                                               //  LDPC-
            return 892;
        case TMTX_CODER_SEL_TURBO:                                              //  Turbo- 
            return 223;
        case TMTX_CODER_SEL_RS:                                                 //  RS- 
            if (((TMTX->CODERS_CONFIG >> 16) & 1) == 0)
                return 239;
            else
                return 223;
    }
    return 0;
}  

unsigned int BitFrameSizeOut()
{
    switch (TMTX->CODER_SEL & 0x3)          
    {
        case 0:
            return FRAME_SIZE_NO_CODING * 8;  
        case TMTX_CODER_SEL_LDPC:                                               //  LDPC-
            if (((TMTX->CODERS_CONFIG >> 24) & 1) == 0)
                return 8160;
            else
                return 8176;
        case TMTX_CODER_SEL_TURBO:                                              //  Turbo- 
            switch ((TMTX->CODERS_CONFIG >> 8) & 3)
            {
                case 0:
                    return 3576;
                case 1:
                    return 5364;
                case 2:
                    return 7152;
                case 3: 
                    return 10728;
            }
        case TMTX_CODER_SEL_RS:                                                 //  RS- 
            return 255*8;
    }
    return 0;
}

/**
 * @brief  
 */  
void SendingProcess()
{  
    TMTX->MARKER_AND_FRAME_SIZE = (192<<16) | BitFrameSizeOut();                //     
    TMTX->MARKER1 = 0xFFFFFFFF;                                                 //   1
    TMTX->MARKER2 = 0x77777777;                                                 //   2
    TMTX->MARKER3 = 0xBBBBBBBB;                                                 //   3
    TMTX->MARKER4 = 0x33333333;                                                 //   4
    TMTX->MARKER5 = 0xDDDDDDDD;                                                 //   5
    TMTX->MARKER6 = 0x55555555;                                                 //   6
    FramesTotalCount = 10;
    FramesSendedCount = 2;                    
      
    c = 1;
    Frame_size = GetFrameSize();                                                //   
        
    if (FramesTotalCount >= 1)
    {
        TMTX->DSCR1_PTR = (unsigned int)&frame1[0];                             //  1:     
        INIT_FRAME(1);
    }
    if (FramesTotalCount >= 2)
    {
        TMTX->DSCR2_PTR = (unsigned int)&frame2[0];                             //  2:    
        INIT_FRAME(2);
    }
           
    tmtx_cur_dscr = 0;
    Irq_counter = 0;
        
    TMTX->GLOBAL_ENABLE=TMTX_GLOBAL_ENABLE;                                     //   ,       
    for(int i=0; i<100; i++);
    TMTX->PHY_FIFO_UNDERRUN=TMTX_PHY_FIFO_UNDERRUN;                             //    FIFO

    while(1)
    {        
        if (((TMTX->PHY_FIFO_UNDERRUN&TMTX_PHY_FIFO_UNDERRUN)==1))              //    FIFO
        {            
            if (FramesSendedCount < FramesTotalCount+2)
                {                                
                     PRINT("\tERROR: FIFO was empty in wrong time\n");
                     DBG_TX_Flush();                                            //   ,    ,   UART
                     tmtx_test_failed = 1;                                
                     break;
                }
                else
                {                                           
                     PRINT("\tTest progress: ok\n");
                     DBG_TX_Flush();                                            //   ,    ,   UART
                     break;
                }
        }
        tmtx->MARKER6 = 0;
    };
    TMTX->GLOBAL_ENABLE=0x00;   
}

/**
 * @brief  
 */  
void TMTX_IRQ (void)
{          
    if (tmtx_cur_dscr == 0)
    {
        if (FramesSendedCount < FramesTotalCount)
        {                  
            INIT_FRAME(1);
        }
        tmtx_cur_dscr = 1;
    }
    else
    {          
        if (FramesSendedCount < FramesTotalCount)
        {               
            INIT_FRAME(2);
        }                
                
        tmtx_cur_dscr = 0;
    }
    FramesSendedCount++;
    Irq_counter++;             
}

/**
 * @brief       TMTX
 */  
void TMTX_Clock_init()
{
    PWR_CLK_Enable(TMTX_CTRL_NUM);                                              //     
    PWR_RST_Disable(TMTX_CTRL_NUM);                                             //     
}

/**
 * @brief  GPIO     TMTX 
 */  
void TMTX_Gpio_Init()
{
    CMN_REG->ALT_FUNCTION_CTRL[6] |= 0;                                         //  GPIO_G
    GPIO_G->ALTFUNCSET = (1<<12) | (1<<13) | (1<<14);                           //  GPIO
}

/**
 * @brief  TMTX
 */ 
T_TEST_RESULT Test_TMTX()
{
    PRINT("\n********** TMTX test start **********\n"); 
    DBG_TX_Flush();                                                             //   ,    ,   UART
      
    TMTX_Gpio_Init();                                                           //  GPIO        
    TMTX_Clock_init();                                                          //                                                     
        
#ifdef CCSDS_TERMINAL
    PRINT("COMMAND=TMTX_START\n");
    DBG_TX_Flush();                                                             //   ,    ,   UART
#       ifdef EXT
    for(unsigned int i=0; i<1000000; i++);
#       else
    for(unsigned int i=0; i<6000000; i++);
#       endif  
#endif        
    unsigned int errors = 0;
           
    TMTX->GLOBAL_ENABLE = 0x00;                                                 //      (!)
    tmtx_test_failed = 0;
    NVIC_EnableIRQ(TMTX_NUM,18);                                                //  
        
    for (int k = 0; k < 10000000; k++);

    PRINT("\tsubtest#0 Codec=none, Randomizer=OFF, Convolution rate=none\n");
    DBG_TX_Flush();                                                             //   ,    ,   UART
    TMTX->CODER_SEL = 0;                    TMTX->CODERS_CONFIG = 0;            //          
    SendingProcess();                                                           //   
    if (tmtx_test_failed)
    {
        errors++;
        goto tmtx_test_end;
    }        
        
    PRINT("               subtest#1 Codec=LDPC_7136, Randomizer=ON, Convolution rate=1/2\n");
    DBG_TX_Flush();                                                             //   ,    ,   UART
    TMTX->CODER_SEL = TMTX_CODER_SEL_LDPC;        TMTX->CODERS_CONFIG = 0<<24;  // LDPC 7136 
    TMTX->CODER_SEL |= 1 << 3;                                                  // Randomizer ON 
    TMTX->CODER_SEL |= 1 << 2;      TMTX->CODERS_CONFIG |= 1;                   // Conv ON, rate 1/2  
    SendingProcess();                                                           //  
    if (tmtx_test_failed)
    {
        errors++;
        goto tmtx_test_end;
    }
        
    PRINT("               subtest#2 Codec=LDPC_7156, Randomizer=OFF, Convolution rate=2/3\n");
    DBG_TX_Flush();                                                             //   ,    ,   UART
    TMTX->CODER_SEL = TMTX_CODER_SEL_LDPC;            
    TMTX->CODERS_CONFIG = 1<<24;                                                // LDPC 7156
    TMTX->CODER_SEL |= 1 << 2;      TMTX->CODERS_CONFIG |= 2;                   // Conv ON, rate 2/3
    SendingProcess();                                                           //  
    if (tmtx_test_failed)
    {
        errors++;
        goto tmtx_test_end;
    }
        
    PRINT("               subtest#3 Codec=Turbo_1/2, Randomizer=ON, Convolution rate=3/4\n");
    DBG_TX_Flush();                                                             //   ,    ,   UART
    TMTX->CODER_SEL = TMTX_CODER_SEL_TURBO;       TMTX->CODERS_CONFIG = 0<<8;   // Turbo 1/2
    TMTX->CODER_SEL |= 1 << 3;                                                  // Randomizer ON 
    TMTX->CODER_SEL |= 1 << 2;      TMTX->CODERS_CONFIG |= 3;                   // Conv ON, rate 3/4
    SendingProcess();                                                           //  
    if (tmtx_test_failed)
    {
        errors++;
        goto tmtx_test_end;
    }
        
    PRINT("               subtest#4 Codec=Turbo_1/3, Randomizer=OFF, Convolution rate=5/6\n");
    DBG_TX_Flush();                                                             //   ,    ,   UART
    TMTX->CODER_SEL = TMTX_CODER_SEL_TURBO;       TMTX->CODERS_CONFIG = 1<<8;   // Turbo 1/3
    TMTX->CODER_SEL |= 1 << 2;      TMTX->CODERS_CONFIG |= 5;                   // Conv ON, rate 5/6
    SendingProcess();                                                           //  
    if (tmtx_test_failed)
    {
        errors++;
        goto tmtx_test_end;
    }              
        
    PRINT("               subtest#5 Codec=Turbo_1/4, Randomizer=ON, Convolution rate=7/8\n");
    DBG_TX_Flush();                                                             //   ,    ,   UART
    TMTX->CODER_SEL = TMTX_CODER_SEL_TURBO;       TMTX->CODERS_CONFIG = 2<<8;   // Turbo 1/4
    TMTX->CODER_SEL |= 1 << 3;                                                  // Randomizer ON
    TMTX->CODER_SEL |= 1 << 2;      TMTX->CODERS_CONFIG |= 7;                   // Conv ON, rate 7/8 
    SendingProcess();                                                           //  
    if (tmtx_test_failed)
    {
        errors++;
        goto tmtx_test_end;
    }
               
    PRINT("               subtest#6 Codec=Turbo_1/6, Randomizer=OFF, Convolution rate=1/2\n");
    DBG_TX_Flush(); 
    TMTX->CODER_SEL = TMTX_CODER_SEL_TURBO;            TMTX->CODERS_CONFIG = 3<<8;   //Turbo 1/6
    TMTX->CODER_SEL |= 1 << 2;      TMTX->CODERS_CONFIG |= 1;             //Conv ON, rate 1/2
    SendingProcess();                                                           //   ,    ,   UART
    if (tmtx_test_failed)
    {
        errors++;
        goto tmtx_test_end;
    }
      
    PRINT("               subtest#7 Codec=RS_239, Randomizer=ON, Convolution rate=2/3\n");
    DBG_TX_Flush();                                                             //   ,    ,   UART
    TMTX->CODER_SEL = TMTX_CODER_SEL_RS;  TMTX->CODERS_CONFIG = (0<<16) | (DUAL_BASIS << 17);  //RS 239
    TMTX->CODER_SEL |= 1 << 3;                                                  // Randomizer ON
    TMTX->CODER_SEL |= 1 << 2;      TMTX->CODERS_CONFIG |= 2;                   // Conv ON, rate 2/3
    SendingProcess();                                                           //  
    if (tmtx_test_failed)
    {
        errors++;
        goto tmtx_test_end;
    }
      
    PRINT("               subtest#8 Codec=RS_223, Randomizer=OFF, Convolution rate=3/4\n");
    DBG_TX_Flush();                                                             //   ,    ,   UART
    TMTX->CODER_SEL = TMTX_CODER_SEL_RS;     TMTX->CODERS_CONFIG = (1<<16) | (DUAL_BASIS << 17);  //RS 223
    TMTX->CODER_SEL |= 1 << 2;      TMTX->CODERS_CONFIG |= 3;                   // Conv ON, rate 3/4
    SendingProcess();                                                           //  
    if (tmtx_test_failed)
    {
        errors++;
        goto tmtx_test_end;
    }
        
#ifdef CCSDS_TERMINAL
         
    for(unsigned int i=0; i<1000000; i++);
    PRINT("COMMAND=TMTX_COMPLETE\n"); 
    DBG_TX_Flush();                                                             //   ,    ,   UART
#endif
        
    PWR_CLK_Disable(TMTX_CTRL_NUM);                                             //     
    PWR_RST_Enable(TMTX_CTRL_NUM);                                              //     
        
tmtx_test_end:
        
    PRINT("\tTMTX test complete. Errors count = %d\n", errors);      
    DBG_TX_Flush();                                                             //   ,    ,   UART
      
    if (errors > 0)
        return RESULT_FAIL;
    else        
        return RESULT_OK;  
}
#endif //TEST_TMTX
