/*!\file
 * \brief    ,            .
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#ifndef TCRX_H_
#define TCRX_H_

#include "types.h"

#define TCRX_DSC_COUNT    16                                                    /**<      */

/** \brief       
 */
typedef struct
{
    volatile uint32_t DSCR_PTR;                                                 /**<       ,     .       4  ,  2   -  */
    volatile uint32_t DSCR_FLAG_SIZE;
}TCRX_DSC_T;

/** \brief          
 */
typedef struct
{
    TCRX_DSC_T DSCR_TBL[TCRX_DSC_COUNT];                                        /**< : 0x000  , 16  (2    ) */
    __RW uint32_t   DSCR_CURRENT;                                               /**< : 0x080    */
    __RW uint32_t   GLOBAL_ENABLE;                                              /**< : 0x084        */
    __RW uint32_t   CODER_SEL;                                                  /**< : 0x088    */
    __RW uint32_t   PHY_FIFO_OVERFLOW;                                          /**< : 0x08C    FIFO  */
    __RW uint32_t   START_SEQ;                                                  /**< : 0x090    */
    __RW uint32_t   TAIL_SEQ1;                                                  /**< : 0x094   ,    */
    __RW uint32_t   TAIL_SEQ2;                                                  /**< : 0x098   ,   */
}TCRX_T;

/**
 * @brief   DSCR_PTR 
 */
#define DSCR_PTR(x)          ((uint32_t)((x & 0x3FFFFFFF) << 0))                /**<       ,     . 
                                                                                          4  ,  2   - */


/**
 * @brief   DSCR_FLAG_SIZE 
 */
#define TCRX_DSC_DSCR_FLAG_SIZE_SIZE(x)          ((uint32_t)((x & 0x3FF) << 0)) /**<      (0x000 = 1024 ) */
#define TCRX_DSC_DSCR_FLAG_SIZE_SOF              ((uint32_t)(1 << 16))          /**< , ,         */
#define TCRX_DSC_DSCR_FLAG_SIZE_EOF              ((uint32_t)(1 << 17))          /**< , ,         */
#define TCRX_DSC_DSCR_FLAG_SIZE_ERROR            ((uint32_t)(1 << 18))          /**< , ,       ,    - */
#define TCRX_DSC_DSCR_FLAG_SIZE_READY            ((uint32_t)(1 << 31))          /**<     */

/**
 * @brief   DSCR_CURRENT
 */
#define TCRX_DSCR_CURRENT_NUM(x)         ((uint32_t)((x & 0xF) << 0))           /**<    */
#define TCRX_DSCR_CURRENT_WORK_FLAG           ((uint32_t)(1 << 31))             /**<   1,       */

/**
 * @brief   GLOBAL_ENABLE
 */
#define TCRX_GLOBAL_ENABLE_EN            ((uint32_t)(1 << 0))                   /**<   ,     */

/**
 * @brief   CODER_SEL
 */
typedef enum 
{
    TCRX_CODER_SEL_RAND_SEL = ((uint32_t)(1 << 0)),                             /**<   */
    TCRX_CODER_SEL_BCH_SEL  = ((uint32_t)(1 << 1)),                             /**<    */
}TCRX_CODER_SEL_T;

/**
 * @brief   PHY_FIFO_OVERFLOW
 */
#define TCRX_PHY_FIFO_OVERFLOW           ((uint32_t)(1 << 0))                   /**<      FIFO */

/**
 * @brief     START_SEQ
 */
#define TCRX_START_SEQ_DATA(x)           ((uint32_t)((x & 0xFFFF) << 0))        /**<    */

typedef enum 
{
    TCRX_START_SEQ_ALLOW_1_ERR = ((uint32_t)(1 << 31)),                         /**<        */

} TCRX_START_SEQ_T;

/**
 * @brief ,     
 */
#define TCRX_DSCR_MAX_SIZE              1024                                    /**<    ( ) */

/**
 * @brief    DSCR_READY, ,   
 * @param index	:  
 * @return  , SET or RESET
 */
FlagStatus TCRX_Dscr_Ready(uint32_t index);

/**
 * @brief   ,  
 * @param index	:  
 * @return   ,  
 */
uint32_t TCRX_Get_Data_Size(uint32_t index);

/**
 * @brief    SOF
 * @param index	:  
 * @return  , SET or RESET
 */
FlagStatus TCRX_SOF_Flag(uint32_t index);

/**
 * @brief    EOF
 * @param index	:  
 * @return  , SET or RESET
 */
FlagStatus TCRX_EOF_Flag(uint32_t index);

/**
 * @brief    ERROR
 * @param index	:  
 * @return  , SET or RESET
 */
FlagStatus TCRX_ERROR_Flag(uint32_t index);

/**
 * @brief     
 * @param index	:  
 */
void TCRX_Set_Current_Descriptor(uint32_t index);

/**
 * @brief    
 * @return  
 */
uint32_t TCRX_Get_Current_Descriptor();


/**
 * @brief   
 */
void TCRX_Enable();

/**
 * @brief   
 */
void TCRX_Disable();

/**
 * @brief   
 * @param randomizer :  ,   :
 *                                              - TCRX_CODER_SEL_RAND
 *                                              - TCRX_CODER_SEL_RAND_OFF    
 * @param bch :   ,   :
 *                                              - TCRX_CODER_SEL_BCH
 *                                              - TCRX_CODER_SEL_BCH_OFF
 * @param viterbi :    ,   :
 *                                              - TCRX_CODER_SEL_VIT_HARD
 *                                              - TCRX_CODER_SEL_VIT_SOFT
 *                                              - TCRX_CODER_SEL_VIT_OFF
 */
void TCRX_Set_Decoder(uint32_t randomizer, uint32_t bch, uint32_t viterbi);

/**
 * @brief     FIFO
 * @return  , SET or RESET
 */
FlagStatus TCRX_FIFO_Overflow();

/**
 * @brief           
 * @param start_seq :    .  16 .       
 * @param allow_1_err :         ,   :
 *                                              - TCRX_START_SEQ_ALLOW_1_ERR
 *                                              - TCRX_START_SEQ_DENY_1_ERR
 */
void TCRX_Set_Start_Sequence(uint32_t start_seq, uint32_t allow_1_err);

/**
 * @brief     
 * @param tail_first :    ,  .  32 .       
 * @param tail_second :    ,  .  32 .       
 */
void TCRX_Set_Tail_Sequence(uint32_t tail_first, uint32_t tail_second);

/**
 * @brief    
 * @param index :  
 * @param pointer :  ,       .     4  
 * @param size :   ,      . (  0  1023)
 */
void TCRX_OpenDescriptor(uint32_t index, uint32_t pointer, uint32_t size);

#endif