/*!\file
 * \brief          .
 *
 * Copyright(C)   , 2015
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#include "chip_5023BC016.h"
#include "uart.h"
#include "debug_uart.h"
#include "ringbuffer.h"
#include "system.h"
#include "gpio.h"

RINGBUFF_T      dbg_uart_tx_rb;                                                 ///<    UART 
uint8_t         dbg_uart_tx_buf[DEBUG_UART_RB_SIZE];                            ///< ,   . 

/**
 * @brief	  UART'
 */
void DBG_UART_Init(void)
{
#ifdef ENABLE_DEBUG_UART
    UART_Init(UART1);                                                           //  UART-  
    UART_SetBaud(UART1, BAUDRATE_921600, 80000000);                             //         
    UART_Config(UART1, 112);/*((!UART_LCR_H_PEN) |                              //    ,      
                             UART_LCR_H_FEN | 
                             UART_LCR_H_WLEN8 | 
                             (UART_LCR_H_SPS_DIS)));*/    
    UART_Enable(UART1);                                                         //   UART
    UART_TXEnable(UART1);                                                       //   
    UART_RXEnable(UART1);
    UART_SetupFIFOS(UART1, UART_IFLS_7DIV8, UART_IFLS_1DIV8);                   //       /
    RingBuffer_Init(&dbg_uart_tx_rb, dbg_uart_tx_buf, 1, DEBUG_UART_RB_SIZE);   //   
    PRINT("__DEBUG_ENABLED");
#endif
}

/**
 * @brief	 ,    ,   UART
 */
void DBG_TX_Process(void)
{
#ifdef ENABLE_DEBUG_UART
    uint8_t ch;
    if (!(UART_GetStatus(DEBUG_UART, UART_FR_BUSY))) 
    {
        if(RingBuffer_Pop(&dbg_uart_tx_rb, &ch))
        {
            UART_SendByte(DEBUG_UART, ch);
        }
    }
#endif
}

/**
 * @brief	  ,    ,   UART
 */
void DBG_TX_Flush(void)
{
#ifdef ENABLE_DEBUG_UART
    uint8_t ch;
    while (!(RingBuffer_IsEmpty(&dbg_uart_tx_rb)))
    {
        if (!(UART_GetStatus(DEBUG_UART, UART_FR_BUSY))) 
        {
            if(RingBuffer_Pop(&dbg_uart_tx_rb, &ch))
            {
                UART_SendByte(DEBUG_UART, ch);
            }
        }
    }
#endif
}

/**
 * @brief	     
 * @param	buffer	:   
 */
void DBG_WriteBuffer(char *buffer)
{
//#ifdef ENABLE_DEBUG_UART
    uint32_t length;
    length = strlen(buffer);
    RingBuffer_InsertMult(&dbg_uart_tx_rb, buffer, length);
//#endif  
}

/**
 * @brief	     UART
 * @param	buffer	:   
 * @param	length	:    
*/
void DBG_ShowBuffer(uint8_t *buffer, uint32_t length)
{
#ifdef ENABLE_DEBUG_UART
    uint32_t i;
    if (length) 
    {
        for(i=0; i<length; i++)
        {
            snprintf(dbg_buffer,256,"%02x ",buffer[i]);
            DBG_WriteBuffer(dbg_buffer);
        }
        DBG_WriteBuffer("\n");
    }
#endif
};

void gpioInit()
{
    GPIO_D->ALTFUNCSET |= (1 << 15); //uart0 tx
    GPIO_E->ALTFUNCSET |= (1 << 0);  //uart0 rx
}

void clockInit()
{
    CMN_REG->PWR_CTRL_CLK |= (1<<2);
    CMN_REG->PWR_CTRL_RST |= (1<<2);
}

/**
 * @brief	    UART'
 * @return	 
*/
uint8_t DBG_GetChar(void)
{
#ifdef ENABLE_DEBUG_UART
    if (!(UART_GetStatus(DEBUG_UART, UART_FR_RXFE))) 
    {
	return (uint8_t) UART_ReadByte(DEBUG_UART);
    }
#endif
    return EOF;
}