/*!\file
 * \brief           .
 * \note  -    ,    52070-2003
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#include "chip_5023BC016.h"
#include "system.h"
#include "mkpd.h"


/**
 * @brief	     
 * @param	pMKPD	:      MKPD
 * @param	invTX	: 1 -    ; 0 -  
 */   
void MKPD_Set_InvTX(MKPD_T *pMKPD, unsigned int invTX)
{
        pMKPD->SETTING2 = (pMKPD->SETTING2 & ~(0x1 << 8)) | MKPD_SETTING2_INVTX(invTX);
}
/**
 * @brief    
 * @param pMKPD:      MKPD
 * @param sys_clk:  ( )
 * @retval None
 */   
void MKPD_Set_System_Clock(MKPD_T *pMKPD, uint32_t sys_clk)
{
    pMKPD->SETTING0 = (pMKPD->SETTING0 & ~(0xFF << 24)) | MKPD_SETTING0_SYS_CLK(sys_clk-1);
}

/**
 * @brief      
 * @param pMKPD:      MKPD
 * @param time:  ( ).    4  15
 * @retval None
 */   
void MKPD_Set_Interpacket_Time(MKPD_T *pMKPD, uint32_t time)
{
    if (time < 4)
    {
        time = 4;
    }
    pMKPD->SETTING1 = (pMKPD->SETTING1 & ~(0xF << 24)) | MKPD_SETTING1_TIME(time);
}
   
/**
 * @brief            
 * @param pMKPD:      MKPD
 * @param rwait:  ( ).    34  255
 * @retval None
 */   
void MKPD_Set_Wait_Time(MKPD_T *pMKPD, uint32_t rwait)
{
    if (rwait < 21)
    {
        rwait = 21;
    }
    pMKPD->SETTING1 = (pMKPD->SETTING1 & ~(0xFF << 16)) | MKPD_SETTING1_RWAIT(rwait);
}

/**
 * @brief      
 * @param pMKPD:      MKPD
 * @param edge:  (   ).    0  255
 * @retval None
 */   
void MKPD_Set_Edge_Time(MKPD_T *pMKPD, uint32_t edge)
{
    pMKPD->SETTING1 = (pMKPD->SETTING1 & ~(0xFF << 8)) | MKPD_SETTING1_EDGE(edge);
}


/**
 * @brief   
 * @param pMKPD:      MKPD
 * @return    . 
 *            :
 *           MKPD_INTERRUPT_BC_INT:     
 *           MKPD_INTERRUPT_EP_INT0:  ,   
 *           MKPD_INTERRUPT_EP_INT1:  ,   
 *           MKPD_INTERRUPT_MON_INT:  ,   
 */   
uint32_t MKPD_Get_Interrupt(MKPD_T *pMKPD)
{
    return pMKPD->INTERRUPT;
}


/**
 * @brief         6  10.
 * @param pMKPD:      MKPD
 * @param index:     (TX_ONE/TX_TWO)  
 * @param com_data:   , 16 
 * @retval None
 */   
void MKPD_Set_BC_COM_Data(MKPD_T *pMKPD, MKPD_TX_T index, uint32_t com_data)
{
    if (index == TX_ONE)
    {
        pMKPD->BCONE2 = (pMKPD->BCONE2 & ~(0xFFFF << 16)) | MKPD_BCONE2_COM_DATA(com_data);
    }
    else
    {
        pMKPD->BCTWO2 = (pMKPD->BCTWO2 & ~(0xFFFF << 16)) | MKPD_BCTWO2_COM_DATA(com_data);
    }
}

/**
 * @brief   
 * @param pMKPD:      MKPD
 * @param index:     (TX_ONE/TX_TWO)  
 * @param type_trans:   (1..10)
 * @retval None
 */   
void MKPD_Set_BC_Transaction_Format(MKPD_T *pMKPD, MKPD_TX_T index, uint32_t type_trans)
{
    if (index == TX_ONE)
    {
        pMKPD->BCONE2 = (pMKPD->BCONE2 & ~(0xF << 0)) | MKPD_BCONE2_TYPE_TRANS((type_trans-1)%10);
    }
    else
    {
        pMKPD->BCTWO2 = (pMKPD->BCTWO2 & ~(0xF << 0)) | MKPD_BCONE2_TYPE_TRANS((type_trans-1)%10);
    }
}





/**
 * @brief ,       
 * @param pMKPD:      MKPD
 * @param index:     (TX_ONE/TX_TWO) 
 * @return SET or RESET -   
 */   
FlagStatus MKPD_BC_IsWaiting(MKPD_T *pMKPD, MKPD_TX_T index)
{
    if (index == TX_ONE)
    {
        return (pMKPD->BCONE2 & MKPD_BCONE2_ENA) ? SET : RESET;
    }
    else
    {
        return (pMKPD->BCTWO2 & MKPD_BCTWO2_ENA) ? SET : RESET;
    }
}

/**
 * @brief ,      
 * @param pMKPD:      MKPD
 * @param index:     (TX_ONE/TX_TWO) 
 * @return SET or RESET -   
 */  
FlagStatus MKPD_BC_IsSending(MKPD_T *pMKPD, MKPD_TX_T index)
{
    if (index == TX_ONE)
    {
        return (pMKPD->BCONE4 & MKPD_BCONE4_STATUS) ? SET : RESET;
    }
    else
    {
        return (pMKPD->BCTWO4 & MKPD_BCTWO4_STATUS) ? SET : RESET;
    }
}

/**
 * @brief     
 * @param pMKPD:      MKPD
 * @param index:     (TX_ONE/TX_TWO) 
 * @return SET or RESET -   
 */  
FlagStatus MKPD_BC_IsError(MKPD_T *pMKPD, MKPD_TX_T index)
{
    if (index == TX_ONE)
    {
        return (pMKPD->BCONE4 & MKPD_BCONE4_ERROR) ? SET : RESET;
    }
    else
    {
        return (pMKPD->BCTWO4 & MKPD_BCTWO4_ERROR) ? SET : RESET;
    }
}

/**
 * @brief     
 * @param pMKPD:      MKPD
 * @param index:     (TX_ONE/TX_TWO) 
 * @return SET or RESET -   
 */    
FlagStatus MKPD_BC_IsTimeout(MKPD_T *pMKPD, MKPD_TX_T index)
{
    if (index == TX_ONE)
    {
        return (pMKPD->BCONE4 & MKPD_BCONE4_TIMEOUT) ? SET : RESET;
    }
    else
    {
        return (pMKPD->BCTWO4 & MKPD_BCTWO4_TIMEOUT) ? SET : RESET;
    }
}

/**
 * @brief   ,   
 * @param pMKPD:      MKPD
 * @param index:     (TX_ONE/TX_TWO) 
 * @return   , 16 
 */   
uint32_t MKPD_BC_Reply_Word(MKPD_T *pMKPD, MKPD_TX_T index)
{
    if (index == TX_ONE)
    {
        return MKPD_BCONE3_REPLY(pMKPD->BCONE3);
    }
    else
    {
        return MKPD_BCTWO3_REPLY(pMKPD->BCTWO3);
    }
}

/**
 * @brief   ,    (   5)
 * @param pMKPD:      MKPD
 * @param index:     (TX_ONE/TX_TWO) 
 * @return   , 16 
 */     
uint32_t MKPD_BC_Reply_Data(MKPD_T *pMKPD, MKPD_TX_T index)
{
    if (index == TX_ONE)
    {
        return MKPD_BCONE3_REPLY_WORD(pMKPD->BCONE3);
    }
    else
    {
        return MKPD_BCTWO3_REPLY_WORD(pMKPD->BCTWO3);
    }
}





/**
 * @brief       
 * @param pMKPD:      MKPD
 * @param rdata:   , 16 
 * @retval None
 */   
void MKPD_Set_Endpoint_RDATA(MKPD_T *pMKPD, uint32_t rdata)
{
    pMKPD->EP1 = (pMKPD->EP1 & ~(0xFFFF << 16)) | MKPD_EP1_RDATA(rdata);
}



/**
 * @brief       
 * @param pMKPD:      MKPD
 * @return   , 16 
 */   
uint32_t MKPD_Get_Endpoint_COM_Data(MKPD_T *pMKPD)
{
    return MKPD_EP3_EP_COM_DATA(pMKPD->EP2);
}

/**
 * @brief    
 * @param pMKPD:      MKPD  
 * @return SET or RESET -    
 */   
FlagStatus MKPD_Get_Endpoint_COM_Timeout(MKPD_T *pMKPD)
{
    return (pMKPD->EP2 & MKPD_EP2_EP_COM_TIMEOUT) ? SET : RESET;
}

/**
 * @brief        (   6, 10)
 * @param pMKPD:      MKPD  
 * @return SET or RESET -    
 */   
FlagStatus MKPD_Get_Endpoint_COM_Error(MKPD_T *pMKPD)
{
    return (pMKPD->EP2 & MKPD_EP2_EP_COM_ERR) ? SET : RESET;
}

/**
 * @brief   ,  
 * @param pMKPD:      MKPD  
 * @return SET or RESET -    
 */  
uint32_t MKPD_Get_Endpoint_Command_Word(MKPD_T *pMKPD)
{
    return MKPD_EP3_EP_COM(pMKPD->BCTWO3);
}

/**
 * @brief    DMA  
 * @param pMKPD:      MKPD  
 * @param dma_monitor_addr:      
 * @retval None
 */   
void MKPD_Set_Monitor_DMA_Address(MKPD_T *pMKPD, uint32_t dma_monitor_addr)
{
        pMKPD->MONITOR0 = dma_monitor_addr;
}

/**
 * @brief    
 * @param pMKPD:      MKPD 
 * @param memory_size:  32- ,   
 * @retval None
 */   
void MKPD_Set_Monitor_Memory_Size(MKPD_T *pMKPD, uint32_t memory_size)
{
    pMKPD->MONITOR1 = memory_size;
}

/**
 * @brief     
 * @param pMKPD:      MKPD 
 * @param threshold:   ,     
 * @retval None
 */   
void MKPD_Set_Monitor_Threshold(MKPD_T *pMKPD, uint32_t threshold)
{
    pMKPD->MONITOR2 = threshold;
}

/**
 * @brief  
 * @param pMKPD:      MKPD 
 * @param address:        
 * @param size:  32- ,   
 * @param threshold:   ,     
 * @retval None
 */
void MKPD_ConfigMonitor(MKPD_T *pMKPD, uint32_t address, uint32_t size, uint32_t threshold)
{        
    MKPD_Set_Monitor_Memory_Size(pMKPD, size);
    MKPD_Set_Monitor_Threshold(pMKPD, threshold);
    MKPD_Set_Monitor_DMA_Address(pMKPD, address);
    MKPD_Set_Mode(pMKPD, MKPD_SETTING0_MODE_M);
}
